/*
Routines to keep track of how long it's been since we sent an USB report.
*/

#include <avr/io.h>
#include <avr/interrupt.h>

volatile unsigned char IdleTimerVal;

//Initialize timer0 to interrupt every 4 ms
void IdleTimerInit(void) {
#if HAVE_IDLE_TIMER
    TCCR0A=2; //CTC, top=OCRA
    TCCR0B=5; //inc every 1024 cycles, which is est. 20KHz.
    OCR0A=80; //int every 4ms
    TIMSK0=2; //enable OCR0A int
#endif
    IdleTimerVal=0; //reset timer
}

//Get amount of 4ms increments since IdleTimerReset
unsigned char IdleTimerGet(void) {
    return IdleTimerVal;
}

//Reset idle timer
void IdleTimerReset(void) {
    IdleTimerVal=0;
}

//Timer0 service routine. Gets called every 4ms
ISR(TIMER0_COMPA_vect) {
    if (IdleTimerVal!=255) IdleTimerVal++;
}
