#include "io.h"
#include "ad.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

/*
IO pins used. Keep in mind: digital buttons read high when not pressed and low
(pulled to gnd) when they are.
*/
#define JLEFTHOR	PC0
#define JLEFTVER	PC1
#define JRIGHTHOR	PC2
#define JRIGHTVER	PC3
#define MUX_CHANA	PC4
#define MUX_CHANB	PC5

#define MUX_A1		PD0
#define J_RIGHTPRESS	PD1
#define MUX_A0		PD2
#define USB_PLUS 	PD3
#define USB_MIN		PD4
#define J_LEFTPRESS	PD5
#define B_BACK		PD6
#define B_START		PD7

#define M_LEFT		PB1
#define M_RIGHT		PB2

#define D_LEFT		PB3
#define D_RIGHT		PB4
#define D_UP		PB5
#define D_DOWN		PB0

/*
Initialize IO
*/
void IoInit(void) {
    //Set pin directions, pull-ups and values.
    DDRB=(1<<M_LEFT)|(1<<M_RIGHT);
    PORTB=(1<<D_RIGHT)|(1<<D_LEFT)|(1<<D_UP)|(1<<D_DOWN);
    DDRD=(1<<MUX_A1)|(1<<MUX_A0)|(1<<J_RIGHTPRESS);
    PORTD=(1<<J_LEFTPRESS)|(1<<B_BACK)|(1<<B_START)|(1<<J_RIGHTPRESS);
    DDRC=0;
    PORTC=0;

    //Reset the USB-lines
    DDRD|=(1<<USB_PLUS)|(1<<USB_MIN);
    _delay_ms(12);
    DDRD&=~((1<<USB_PLUS)|(1<<USB_MIN));
}

/*
Dpad is handled as a hat switch. The direction the unit points at is given
in 45 degrees increments, or 15 if not pressed. This routine will return 
that value.
*/
unsigned char IoGetDpadRot(void) {
    char x,y;
    x=0; y=0;
    //Somehow, hid turns x and y around :?
    if (!(PINB&(1<<D_LEFT))) y=-1;
    if (!(PINB&(1<<D_RIGHT))) y=1;
    if (!(PINB&(1<<D_UP))) x=-1;
    if (!(PINB&(1<<D_DOWN))) x=1;
    
    if (x==1) {
	if (y==-1) return 1;
	if (y==1) return 3;
	return 2;
    } else if (x==-1) {
	if (y==1) return 5;
	if (y==-1) return 7;
	return 6;
    } else { //x==0
	if (y==-1) return 0;
	if (y==1) return 4;
	return 15;
    }
}

//Position of the buttons in the returned report
#define POS_A 0
#define POS_B 1
#define POS_X 2
#define POS_Y 3
#define POS_BLACK 4
#define POS_WHITE 5
#define POS_START 6
#define POS_BACK 7
#define POS_THUMBR 8
#define POS_THUMBL 9

//Buttons a,b,x,y,white,black are in fact analog (pressure sensitive). The 
//following define sets the threshold after we think the button is actually
//pressed.
#define ANALOG_BUTTON_THR 900

int IoGetButtons(void) {
    int b=0;
    //Analog buttons. AdChannel is filled in ad.c by the interrupt service there.
    if (AdChannel[CHAN_BA]<ANALOG_BUTTON_THR) b|=(1<<POS_A);
    if (AdChannel[CHAN_BB]<ANALOG_BUTTON_THR) b|=(1<<POS_B);
    if (AdChannel[CHAN_BX]<ANALOG_BUTTON_THR) b|=(1<<POS_X);
    if (AdChannel[CHAN_BY]<ANALOG_BUTTON_THR) b|=(1<<POS_Y);
    if (AdChannel[CHAN_BBLACK]<ANALOG_BUTTON_THR) b|=(1<<POS_BLACK);
    if (AdChannel[CHAN_BWHITE]<ANALOG_BUTTON_THR) b|=(1<<POS_WHITE);
    //digital ones
    if (!(PIND&(1<<B_START))) b|=(1<<POS_START);
    if (!(PIND&(1<<B_BACK))) b|=(1<<POS_BACK);
    if (!(PIND&(1<<J_LEFTPRESS))) b|=(1<<POS_THUMBL);
    if (!(PIND&(1<<J_RIGHTPRESS))) b|=(1<<POS_THUMBR);    
    return b;
}

/*
Set the xpad mux to the address contained in 'a'.
*/
void IoSetMuxAddr(char a) {
    if (a&1) {
	PORTD|=(1<<MUX_A0);
    } else {
	PORTD&=~(1<<MUX_A0);
    }
    if (a&2) {
	PORTD|=(1<<MUX_A1);
    } else {
	PORTD&=~(1<<MUX_A1);
    }
}
