/*
The routines here build a report containing the values of the buttons and axes,
according to the format defined in report_desc.c.
*/
#include "ad.h"
#include "io.h"
#include "report.h"

//Helper routine: Put an 16-bit integer in usb order into 2 chars
static void PutInt(unsigned char* where, int data) {
    where[0]=(data&0xff);
    where[1]=(data>>8);
    return;
}

/*
This routine builds the report into the buffer passed to it. The buffer needs
to be long enough to allow the report to fit into it.
*/
void BuildReport(unsigned char* report) {
    int t;
    //Clear out buffer first, for good measure
    for (t=0; t<REPORT_BUFFER_LEN; t++) report[t]=0;
    //Put the various values. The joystick axes should go from -32k to 32k,
    //instead of the 0-1024 that is in AdChannel.
    PutInt(&report[0],(AdChannel[CHAN_LX]-512)<<6); 
    PutInt(&report[2],(AdChannel[CHAN_LY]-512)<<6);
    PutInt(&report[4],-((AdChannel[CHAN_RX]-512)<<6));
    PutInt(&report[6],-((AdChannel[CHAN_RY]-512)<<6));
    //The 2 triggers shoudl go from 0 to 32K
    t=30000-(AdChannel[CHAN_LTRIG]<<5); if (t<0) t=0;
    PutInt(&report[8],t);
    t=30000-(AdChannel[CHAN_RTRIG]<<5); if (t<0) t=0;
    PutInt(&report[10],t);
    //IoGetButtons should put the buttons in the right order already
    t=IoGetButtons();
    PutInt(&report[12],t);
    //Patch the D-pad hatch value in.
    report[13]|=(IoGetDpadRot()<<2);
    return;
}
